﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Drawing.Drawing2D;

namespace howto_vector_angle
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private List<PointF> Points = new List<PointF>();

        private void picCanvas_MouseClick(object sender, MouseEventArgs e)
        {
            if (Points.Count > 2) Points = new List<PointF>();
            Points.Add(e.Location);
            picCanvas.Refresh();

            if (Points.Count == 3)
            {
                double radians = VectorAngle(
                    Points[0], Points[1],
                    Points[1], Points[2]);
                double degrees = radians / Math.PI * 180;
                lblAngle.Text = radians.ToString("0.00") + " radians, " +
                    degrees.ToString("0.00") + " degrees";
            }
            else
                lblAngle.Text = "";
        }

        private const int Radius = 4;

        private void picCanvas_Paint(object sender, PaintEventArgs e)
        {
            e.Graphics.SmoothingMode = SmoothingMode.AntiAlias;

            if (Points.Count > 1)
            {
                e.Graphics.DrawLines(Pens.Red, Points.ToArray());
                for (int i = 1; i < Points.Count; i++)
                    DrawArrowhead(e.Graphics, Pens.Red, Radius,
                        Points[i - 1], Points[i]);
            }

            foreach (PointF point in Points)
            {
                RectangleF rect = new RectangleF(
                    point.X - Radius, point.Y - Radius,
                    2 * Radius, 2 * Radius);
                e.Graphics.FillEllipse(Brushes.LightBlue, rect);
                e.Graphics.DrawEllipse(Pens.Blue, rect);
            }
        }

        // Return the angle between vector p11 --> p12 and p21 --> p22.
        // Angles less than zero are to the left. Angles greater than
        // zero are to the right.
        private double VectorAngle(PointF p11, PointF p12, PointF p21, PointF p22)
        {
            // Find the vectors.
            PointF v1 = new PointF(p12.X - p11.X, p12.Y - p11.Y);
            PointF v2 = new PointF(p22.X - p21.X, p22.Y - p21.Y);

            // Calculate the vector lengths.
            double len1 = Math.Sqrt(v1.X * v1.X + v1.Y * v1.Y);
            double len2 = Math.Sqrt(v2.X * v2.X + v2.Y * v2.Y);

            // Use the dot product to get the cosine.
            double dot_product = v1.X * v2.X + v1.Y * v2.Y;
            double cos = dot_product / len1 / len2;

            // Use the cross product to get the sine.
            double cross_product = v1.X * v2.Y - v1.Y * v2.X;
            double sin = cross_product / len1 / len2;

            // Find the angle.
            double angle = Math.Acos(cos);
            if (sin < 0) angle = -angle;
            return angle;
        }

        // Draw an arrowhead at the end of the segment.
        private void DrawArrowhead(Graphics gr, Pen pen,
            float radius, PointF p1, PointF p2)
        {
            float dx = p2.X - p1.X;
            float dy = p2.Y - p1.Y;
            float dist = (float)Math.Sqrt(dx * dx + dy * dy);
            dx /= dist;
            dy /= dist;
            float ux = dx;  // Unit vector.
            float uy = dy;
            const float scale = 6;
            dx *= scale;
            dy *= scale;
            float p1x = -dy;
            float p1y = dx;
            float p2x = dy;
            float p2y = -dx;

            float cx = p2.X - ux * radius;
            float cy = p2.Y - uy * radius;
            PointF[] points =
            {
                new PointF(cx - dx + p1x, cy - dy + p1y),
                new PointF(cx, cy),
                new PointF(cx - dx + p2x, cy - dy + p2y),
            };
            gr.DrawLines(pen, points);
        }
    }
}
